//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Filters.h"
#pragma package(smart_init)
////////////////////////////////////////////////////////////////////////////////
TLowPassFilter::TLowPassFilter(float Fc)
{ // Fc -    0.... 0,5
   N= fabs(1.0/Fc * 2.0 *M_PI);
   if(N>1000)N=1000;
   if(N%2==0)N=N+1;
   Z=new float[N+1];
   B=new float[N+1];
   memset(Z,0.0,N*sizeof(float)); memset(B,0.0,N*sizeof(float));
   //  -
   float Ti=N*0.15;   //0.3
    float G=0.0;
    float betta=0.1;
   for(int n=0; n<N; n++)
   {
    float t=n-N/2;
    //B[n]=0.42+0.5*cos(TWOPI*t/N)+0.08*cos(2*TWOPI*t/N);   // -
    B[n]=cos(M_PI*betta*t/N)/(1.0-4*betta*betta*t*t/N/N);
    //B[n]=exp(-2*(t/Ti)*(t/Ti));
    G+=fabs(B[n]);
   }
   //G=10;
   if(G<0.0000001)G=1;
   for(int n=0; n<N; n++) B[n]=B[n]/G;
}
//------------------------------------------------------------------------------
float TLowPassFilter::filter(float s)
{ //    
  //   
    memmove(&Z[1],Z,(N-1)*sizeof(float));
    Z[0]=s;
    float sum=0.0;
    for(int k=0; k<N; k++) sum+=Z[k]*B[k];
    return sum;
}
//------------------------------------------------------------------------------
int TLowPassFilter::filter(float s[], int n)
{   //   
    //      
   for(int i=0; i<n; i++)
    s[i]=filter(s[i]);
   return n;
}
//------------------------------------------------------------------------------
TLowPassFilter::~TLowPassFilter()
{
  delete[] Z; delete[] B;
}
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
TFilter::TFilter(float F1, float F2)
{ /*   
      
     F1 -   
     F2 -   
  */
   F0=(F1+F2)/2;
   BW=F2-F1;
   N= fabs(3.0*M_PI/(F2-F1));
   if(N>65)N=65;
   
   Z=new float[N+1];
   B=new float[N+1];
   memset(Z,0.0,N*sizeof(float)); memset(B,0.0,N*sizeof(float));

   //  -
   float Ti=N*0.2;   //0.3
   float G=0.0;
   for(int n=0; n<N; n++)
   {
    //B[n]=0.5-0.5*cos(TWOPI*n/N); // 
    float t=n-N/2;
    //B[n]=exp(-2*(t/Ti)*(t/Ti));  // 
    B[n]=0.42+0.5*cos(TWOPI*t/N)+0.08*cos(2*TWOPI*t/N);   // -
    
    B[n]=B[n]*sin(TWOPI*F0*n);
    G+=fabs(B[n]);
   }
   for(int n=0; n<N; n++) B[n]=B[n]/G;
}
//------------------------------------------------------------------------------
TFilter::~TFilter()
{
  delete[] Z; delete[] B;
}
//------------------------------------------------------------------------------
void TFilter::filter(float * x, int n, float * y)
{ /*  
     x[] -  
     n   -  
     y[] -  
  */
  for(int i=0; i<n; i++)
  {
    memmove(&Z[1],Z,(N-1)*sizeof(float));
    Z[0]=x[i];
    float sum=0.0;
    for(int k=0; k<N; k++) sum+=Z[k]*B[k];
    y[i]=sum;
  }
}
//------------------------------------------------------------------------------
float TFilter::filter(float x)
{ /*  
     x[] -  
     n   -  
     y[] -  
  */
    memmove(&Z[1],Z,(N-1)*sizeof(float));
    Z[0]=x;
    float sum=0.0;
    for(int k=0; k<N; k++) sum+=Z[k]*B[k];
    return sum;
}
//------------------------------------------------------------------------------
float TFilter::getF0(void)
{
     return (F0);
}
//------------------------------------------------------------------------------
float TFilter::getBW(void)
{
     return (BW);
}
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////



//---------------------------------------------------------------------------
TCosBPFilter::TCosBPFilter(float F1, float F2, float betta)
{  /*      
         BETTA

    */
   N=fabs(1.0/(F2-F1) * M_PI);  ////?????
   if(N>1000)N=1000;
   Z=new float[N+1];
   B=new float[N+1];
   memset(Z,0.0,N*sizeof(float)); memset(B,0.0,N*sizeof(float));

   float G=0.0;
   for(int n=0; n<N; n++)
   {
    float t=n-N/2;
    if(t==0) B[n]=1.0;
    else B[n]= (sin(M_PI*t/N)/(M_PI*t/N)) * (cos(M_PI*betta*t/N)/(1.0-4*betta*betta*t*t/N/N));
    G+=fabs(B[n]);
   }
   G=10;
   for(int n=0; n<N; n++) B[n]=B[n]/G;

}
//------------------------------------------------------------------------------
TCosBPFilter::~TCosBPFilter()
{
  delete[] Z; delete[] B;
}
//------------------------------------------------------------------------------
void TCosBPFilter::filter(float * x, int n, float * y)
{ /*  
     x[] -  
     n   -  
     y[] -  
  */
  for(int i=0; i<n; i++)
  {
    memmove(&Z[1],Z,(N-1)*sizeof(float));
    Z[0]=x[i];
    float sum=0.0;
    for(int k=0; k<N; k++) sum+=Z[k]*B[k];
    y[i]=sum;
  }
}
//------------------------------------------------------------------------------
float TCosBPFilter::filter(float x)
{ /*  
     x[] -  
     n   -  
     y[] -  
  */
    memmove(&Z[1],Z,(N-1)*sizeof(float));
    Z[0]=x;
    float sum=0.0;
    for(int k=0; k<N; k++) sum+=Z[k]*B[k];
    return sum;
}
//------------------------------------------------------------------------------
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////   ////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
TAdaptiveFilter::TAdaptiveFilter(int n, float mu)
{
  N=n; MU=mu; E=0.0;
  Z=new float[N+1];
  B=new float[N+1];
  B[0]=1.0;
}
//------------------------------------------------------------------------------
float TAdaptiveFilter::filter(float x)
{  float y=0.0;
   memmove(&Z[1],Z,(N-1)*sizeof(float));
   Z[0]=x;
   for(int i=0;i<N;i++) y+=Z[i]*B[i];
   //E=
   return y;
}
//------------------------------------------------------------------------------
TAdaptiveFilter::~TAdaptiveFilter()
{
  delete[] Z; delete[] B;
}
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
TIIRLPF::TIIRLPF(float Fs, int Nz)
{ /*
     Fs -   
     Nz - - 
   */
   float alfa=exp(-2*M_PI*Fs);
   A=1.0-alfa;
   B=alfa;
   N=Nz;
   Y=new float[N];
   memset(Y,0,N*sizeof(float));
}
//------------------------------------------------------------------------------
TIIRLPF::~TIIRLPF()
{
  delete[] Y;
}
//------------------------------------------------------------------------------
float TIIRLPF::filter(float x)
{
   float y=0;
   Y[0]=A*x+B*Y[0];
   for(int i=1; i<N;i++) Y[i]=A*Y[i-1]+B*Y[i];
   y=Y[N-1];
   return y;
}
//------------------------------------------------------------------------------
float TIIRLPF::filter(float * x, int n)
{
   float Gain=0;
   for (int i=0; i<n; i++)
   {
      x[i]=filter(x[i]);
   }
   return Gain;
}
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
TIIRBPF2::TIIRBPF2(float Fc, float BW)
{
 // Fc -   
 // BW -    0.707  0...0,5
 F0=Fc;  BW0=BW;
 double K, R;
 R=1-3*BW;
 K=(1-2*R*cos(2*M_PI*Fc)+R*R)/(2-2*cos(2*M_PI*Fc));
 A0=1-K;
 A1=2*(K-R)*cos(2*M_PI*Fc);
 A2=R*R-K;
 B1=2*R*cos(2*M_PI*Fc);
 B2=-R*R;
 X2=X1=X0=0;
 Y2=Y1=Y0=0;
}
//------------------------------------------------------------------------------
float TIIRBPF2::filter(float x)
{
 X2=X1; X1=X0; X0=x;
 Y2=Y1; Y1=Y0;
 Y0=A0*X0+A1*X1+A2*X2+B1*Y1+B2*Y2;
 return Y0;
}
//------------------------------------------------------------------------------
float TIIRBPF2::filter(float x[], int n, float y[])
{
 for(int i=0; i<n; i++) y[i]=filter(x[i]);
 return y[n-1];
}
//------------------------------------------------------------------------------
float TIIRBPF2::getF0(void)
{
     return (F0);
}
//------------------------------------------------------------------------------
float TIIRBPF2::getBW(void)
{
     return (BW0);
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
#define NCoef 18
//  DCF39    135500 ( )
float iir3330(float NewSample) {
    double ACoef[NCoef+1] = {
        0.00000000362112750018,
        0.00000000000000000000,
        -0.00000003259014750158,
        0.00000000000000000000,
        0.00000013036059000633,
        0.00000000000000000000,
        -0.00000030417471001476,
        0.00000000000000000000,
        0.00000045626206502214,
        0.00000000000000000000,
        -0.00000045626206502214,
        0.00000000000000000000,
        0.00000030417471001476,
        0.00000000000000000000,
        -0.00000013036059000633,
        0.00000000000000000000,
        0.00000003259014750158,
        0.00000000000000000000,
        -0.00000000362112750018
    };

    double BCoef[NCoef+1] = {
        1.00000000000000000000,
        3.06175136445556230000,
        12.87994115625079800000,
        27.05438192537606000000,
        63.84148706559923900000,
        100.67104450230769000000,
        167.91338890970209000000,
        206.82933486176728000000,
        263.54260477757776000000,
        257.12922417247586000000,
        257.98434330743657000000,
        198.19468786622434000000,
        157.50977255172643000000,
        92.43839364563587000000,
        57.38380055886301500000,
        23.80289371688397500000,
        11.09280089896825500000,
        2.58090341137965760000,
        0.82517712991430703000
    };

    static double y[NCoef+1]; //output samples
    static double x[NCoef+1]; //input samples
    int n;

    //shift the old samples
    for(n=NCoef; n>0; n--) {
       x[n] = x[n-1];
       y[n] = y[n-1];
    }

    //Calculate the new output
    x[0] = NewSample;
    y[0] = ACoef[0] * x[0];
    for(n=1; n<=NCoef; n++)
        y[0] += ACoef[n] * x[n] - BCoef[n] * y[n];
    
    return y[0];
}

//  DCF39    135500 ( )
float iir3670(float NewSample) {
    double ACoef[NCoef+1] = {
        0.00000001116718489636,
        0.00000000000000000000,
        -0.00000010050466406722,
        0.00000000000000000000,
        0.00000040201865626889,
        0.00000000000000000000,
        -0.00000093804353129408,
        0.00000000000000000000,
        0.00000140706529694112,
        0.00000000000000000000,
        -0.00000140706529694112,
        0.00000000000000000000,
        0.00000093804353129408,
        0.00000000000000000000,
        -0.00000040201865626889,
        0.00000000000000000000,
        0.00000010050466406722,
        0.00000000000000000000,
        -0.00000001116718489636
    };

    double BCoef[NCoef+1] = {
        1.00000000000000000000,
        6.11996769060054560000,
        25.36851888272446200000,
        73.92029145697928500000,
        174.04942567866155000000,
        334.23660030454641000000,
        547.35676898602094000000,
        765.24488059541159000000,
        932.17465287590619000000,
        985.09520939996332000000,
        912.49675208642998000000,
        733.27565141745447000000,
        513.41521223960876000000,
        306.88798637775164000000,
        156.43156043967298000000,
        65.03314116490669500000,
        21.84679229454832600000,
        5.15882696216344700000,
        0.82517712991929848000
    };

    static double y[NCoef+1]; //output samples
    static double x[NCoef+1]; //input samples
    int n;

    //shift the old samples
    for(n=NCoef; n>0; n--) {
       x[n] = x[n-1];
       y[n] = y[n-1];
    }

    //Calculate the new output
    x[0] = NewSample;
    y[0] = ACoef[0] * x[0];
    for(n=1; n<=NCoef; n++)
        y[0] += ACoef[n] * x[n] - BCoef[n] * y[n];
    
    return y[0];
}



TIIR_DCFHGA::TIIR_DCFHGA(float Fc, float Bw)
{
  F0=Fc; BW0=Bw;
}

//------------------------------------------------------------------------------
float TIIR_DCFHGA::filter(float x)
{
   float y=0;
   switch((int)F0)
   { case 3330: y=iir3330(x); break;
     case 3670: y=iir3670(x); break;
   }
   return y;
}
//------------------------------------------------------------------------------
float TIIR_DCFHGA::filter(float * x, int n, float * y)
{
   for (int i=0; i<n; i++)
   {
      y[i]=filter(x[i]);
   }
   return n;
}
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
float TIIR_DCFHGA::getF0(void)
{
     return (F0/12000);
}
//------------------------------------------------------------------------------
float TIIR_DCFHGA::getBW(void)
{
     return (BW0/12000);
}
